<?php

namespace App\Controller;

use App\Entity\User;
use App\Entity\TypeAutorite;
use App\Repository\TypeAutoriteRepository;
use App\Entity\Abonne;
use App\Entity\AutoriteContractante;
use Doctrine\ORM\EntityManagerInterface;
use ApiPlatform\Core\DataPersister\ContextAwareDataPersisterInterface;
use Symfony\Component\Security\Core\Encoder\UserPasswordEncoderInterface;

/**
 *
 */
class UserController implements ContextAwareDataPersisterInterface
{
    private $_entityManager;
    private $_passwordEncoder;

    public function __construct(
        EntityManagerInterface $entityManager,
        UserPasswordEncoderInterface $passwordEncoder
       
    ) {
        $this->_entityManager = $entityManager;
        $this->_passwordEncoder = $passwordEncoder;
        
    }

    /**
     * {@inheritdoc}
     */
    public function supports($data, array $context = []): bool
    {
        return $data instanceof User;
    }
    

    /**
     * @param User $data
     */
    public function persist($data, array $context = [])
    {
        if ($data->getPlainPassword()) {
            $data->setPassword(
                $this->_passwordEncoder->encodePassword(
                    $data,
                    $data->getPlainPassword()
                )
            );
            $data->eraseCredentials();
        }
        $this->_entityManager->persist($data);

        if($data->getRoles()===["ROLE_USER"]){
            $aut=new AutoriteContractante();
            $aut->setLibelle($data->getOrganisation());
            $aut->setUser($data);
            $type = $this->_entityManager->getRepository(TypeAutorite::class)->find($data->getType()); 
            $aut->setTypeAutorite($type);
            $this->_entityManager->persist($aut);
        }else{
            $abonne=new Abonne();
            $abonne->setOrganisation($data->getOrganisation());
            $abonne->setUserId($data);
            $this->_entityManager->persist($abonne);
        }
        
        $this->_entityManager->flush();

       
    }

    /**
     * {@inheritdoc}
     */
    public function remove($data, array $context = [])
    {
        $this->_entityManager->remove($data);
        $this->_entityManager->flush();
    }
}