<?php

namespace App\Entity;

use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Annotation\ApiFilter;
use Symfony\Component\Serializer\Annotation\Groups;
use App\Repository\AppelOffreRepository;
use ApiPlatform\Core\Annotation\ApiResource;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\DateFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\RangeFilter;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\OrderFilter;
use Symfony\Component\HttpFoundation\File\File;
use Vich\UploaderBundle\Mapping\Annotation as Vich;
use App\Controller\DetailsOffreController;


/**
 * @ApiResource(
 *   attributes={
 *    "order"={"publication":"DESC"}
 *    
 *   },
 *   normalizationContext={"groups"={"read:offres"}},
 *   collectionOperations={
 *         "post"={
 *             "controller"=DetailsOffreController::class,
 *             "deserialize"=false,
 *             "path"="/appel_offres/ajout",
 *             "security"="is_granted('ROLE_ADMIN') or is_granted('ROLE_USER') ",
 *             "validation_groups"={"Default", "write:offres"},
 *             "openapi_context"={
 *                 "requestBody"={
 *                     "content"={
 *                         "multipart/form-data"={
 *                             "schema"={
 *                                 "type"="object",
 *                                 "properties"={
 *                                     "file"={
 *                                         "type"="string",
 *                                         "format"="binary"
 *                                     }
 *                                 }
 *                             }
 *                         }
 *                     }
 *                 }
 *             }
 *         },
 *         "get"
 *     },
 *     
 * )
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={"marche":"exact","autorite_contractante":"exact","autorite_contractante.type_autorite":"exact","direction":"exact","libelle"="ipartial","autorite.user.email":"exact"}
 * ),
 * @ApiFilter(
 *   DateFilter::class,
 *   properties={"publication","limite"}
 * ),
 * @ApiFilter(
 *   RangeFilter::class,
 *   properties={"publication"}
 * ),
 * @ORM\Entity(repositoryClass=AppelOffreRepository::class)
 * @Vich\Uploadable
 */
class AppelOffre
{
    /**
     * @ORM\Id
     * @ORM\GeneratedValue
     * @Groups({"read:offres","read:favoris"})
     * @ORM\Column(type="integer")
     * 
    */
    private $id;

    /**
     * @Groups({"read:offres","write:offres"})
     * @ORM\Column(type="string", length=255)
     * 
     */
    private $libelle;

    /**
     * @ORM\Column(type="datetime")
     * @Groups({"read:offres","write:offres"})
     */
    private $publication;

    /**
     * @ORM\Column(type="date")
     * @Groups({"read:offres","write:offres"})
     */
    private $ouverture;

    /**
     * @ORM\Column(type="date")
     * @Groups({"read:offres","write:offres"})
     */
    private $limite;

    /**
     * @ORM\Column(type="string", length=255)
     * @Groups({"read:offres","write:offres"})
     */
    private $lieu;

    /**
     * @ORM\Column(type="string", length=255)
     * @Groups({"read:offres","write:offres"})
     */
    private $ref;

    /**
     * @ORM\ManyToOne(targetEntity=AutoriteContractante::class, inversedBy="appelOffres")
     * @ORM\JoinColumn(nullable=true)
     * @Groups({"read:offres","write:offres"})
     */
    private $autorite;

    /**
     * @ORM\ManyToOne(targetEntity=TypeMarche::class, inversedBy="appelOffres")
     * @Groups({"read:offres","write:offres"})
     */
    private $marche;


    /**
     * @Groups({"read:offres","write:offres"})
     * @ORM\ManyToOne(targetEntity=Direction::class, inversedBy="appelOffres")
     */
    private $direction;

    /** 
    * @Groups({"read:offres","write:offres"})
    * NOTE: This is not a mapped field of entity metadata, just a simple property.
    * 
    * @Vich\UploadableField(mapping="pdf", fileNameProperty="filename",)
    * 
    * @var File|null
    */
   private $file;

   /**
    * @Groups({"read:offres","write:offres"})
    * @ORM\Column(type="string")
    *
    * @var string|null
    */
   private $filename;

    /**
     * @ORM\OneToMany(targetEntity=Favoris::class, mappedBy="offre")
    */
    private $favoris;

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getLibelle(): ?string
    {
        return $this->libelle;
    }

    public function setLibelle(string $libelle): self
    {
        $this->libelle = $libelle;

        return $this;
    }

    public function getPublication(): ?\DateTimeInterface
    {
        return $this->publication;
    }

    public function setPublication(\DateTimeInterface $publication): self
    {
        $this->publication = $publication;

        return $this;
    }

    public function getOuverture(): ?\DateTimeInterface
    {
        return $this->ouverture;
    }

    public function setOuverture(\DateTimeInterface $ouverture): self
    {
        $this->ouverture = $ouverture;

        return $this;
    }

    public function getLimite(): ?\DateTimeInterface
    {
        return $this->limite;
    }

    public function setLimite(\DateTimeInterface $limite): self
    {
        $this->limite = $limite;

        return $this;
    }

    public function getLieu(): ?string
    {
        return $this->lieu;
    }

    public function setLieu(string $lieu): self
    {
        $this->lieu = $lieu;

        return $this;
    }

    public function getRef(): ?string
    {
        return $this->ref;
    }

    public function setRef(string $ref): self
    {
        $this->ref = $ref;

        return $this;
    }

   
    public function getAutorite(): ?AutoriteContractante
    {
        return $this->autorite;
    }

    public function setAutorite(?AutoriteContractante $autorite): self
    {
        $this->autorite = $autorite;

        return $this;
    }

    public function getMarche(): ?TypeMarche
    {
        return $this->marche;
    }

    public function setMarche(?TypeMarche $marche): self
    {
        $this->marche = $marche;

        return $this;
    }

    public function getDirection(): ?Direction
    {
        return $this->direction;
    }

    public function setDirection(?Direction $direction): self
    {
        $this->direction = $direction;

        return $this;
    }

     /**
    * If manually uploading a file (i.e. not using Symfony Form) ensure an instance
    * of 'UploadedFile' is injected into this setter to trigger the update. If this
    * bundle's configuration parameter 'inject_on_load' is set to 'true' this setter
    * must be able to accept an instance of 'File' as the bundle will inject one here
    * during Doctrine hydration.
    *
    * @param File|\Symfony\Component\HttpFoundation\File\UploadedFile|null $file
    */
   public function setFile(?File $file = null): void
   {
       $this->file = $file;
   }

   public function getFile(): ?File
   {
       return $this->file;
   }

   public function setFilename(?string $filename): void
   {
       $this->filename = $filename;
   }

   public function getFilename(): ?string
   {
       return $this->filename;
   }

   /**
    * @return Collection|Favoris[]
    */
   public function getFavoris(): Collection
   {
       return $this->favoris;
   }

   public function addFavori(Favoris $favori): self
   {
       if (!$this->favoris->contains($favori)) {
           $this->favoris[] = $favori;
           $favori->setOffre($this);
       }

       return $this;
   }

   public function removeFavori(Favoris $favori): self
   {
       if ($this->favoris->removeElement($favori)) {
           // set the owning side to null (unless already changed)
           if ($favori->getOffre() === $this) {
               $favori->setOffre(null);
           }
       }
       return $this;
   }
}
