<?php

namespace App\Entity;

use Symfony\Component\Serializer\Annotation\Groups;
use App\Repository\AutoriteContractanteRepository;
use ApiPlatform\Core\Annotation\ApiResource;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Annotation\ApiFilter;


/**
 * @ApiResource(
 *   normalizationContext={"groups"={"read:autorite"}},
 *   itemOperations={"get"}
 * ),
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={"type_autorite":"exact","user.email":"exact"}
 * ),
 * @ORM\Entity(repositoryClass=AutoriteContractanteRepository::class)
 */
class AutoriteContractante
{
    /**
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     * @Groups({"read:offres","read:tautorite","read:autorite","read:directions","write:offres"})
     */
    private $id;

    /**
     * @ORM\Column(type="string", length=255)
     * @Groups({"read:offres","read:tautorite","read:autorite"})
     */
    private $libelle;

    /**
     * @Groups({"read:tautorite","read:autorite"})
     * @ORM\OneToMany(targetEntity=AppelOffre::class, mappedBy="autorite")
     */
    private $appelOffres;

    /**
     * @Groups({"read:offres","read:autorite"})
     * @ORM\ManyToOne(targetEntity=TypeAutorite::class, inversedBy="autoriteContractantes")
     */
    private $type_autorite;

    /**
     * @Groups({"read:autorite"})
     * @ORM\OneToMany(targetEntity=Direction::class, mappedBy="autorite")
     */
    private $directions;

    /**
     * @Groups({"read:offres","read:autorite"})
     * @ORM\ManyToOne(targetEntity=User::class, inversedBy="autoriteContractantes")
     */
    private $user;


    public function __construct()
    {
        $this->appelOffres = new ArrayCollection();
        $this->directions = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getLibelle(): ?string
    {
        return $this->libelle;
    }

    public function setLibelle(string $libelle): self
    {
        $this->libelle = $libelle;

        return $this;
    }

    /**
     * @return Collection|AppelOffre[]
     */
    public function getAppelOffres(): Collection
    {
        return $this->appelOffres;
    }

    public function addAppelOffre(AppelOffre $appelOffre): self
    {
        if (!$this->appelOffres->contains($appelOffre)) {
            $this->appelOffres[] = $appelOffre;
            $appelOffre->setAutoriteContractante($this);
        }

        return $this;
    }

    public function removeAppelOffre(AppelOffre $appelOffre): self
    {
        if ($this->appelOffres->removeElement($appelOffre)) {
            // set the owning side to null (unless already changed)
            if ($appelOffre->getAutoriteContractante() === $this) {
                $appelOffre->setAutoriteContractante(null);
            }
        }

        return $this;
    }

    public function getTypeAutorite(): ?TypeAutorite
    {
        return $this->type_autorite;
    }

    public function setTypeAutorite(?TypeAutorite $type_autorite): self
    {
        $this->type_autorite = $type_autorite;

        return $this;
    }

    /**
     * @return Collection|Direction[]
     */
    public function getDirections(): Collection
    {
        return $this->directions;
    }

    public function addDirection(Direction $direction): self
    {
        if (!$this->directions->contains($direction)) {
            $this->directions[] = $direction;
            $direction->setAutorite($this);
        }

        return $this;
    }

    public function removeDirection(Direction $direction): self
    {
        if ($this->directions->removeElement($direction)) {
            // set the owning side to null (unless already changed)
            if ($direction->getAutorite() === $this) {
                $direction->setAutorite(null);
            }
        }

        return $this;
    }

    public function getUser(): ?User
    {
        return $this->user;
    }

    public function setUser(?User $user): self
    {
        $this->user = $user;

        return $this;
    }

}
