<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Symfony\Component\Serializer\Annotation\Groups;
use App\Repository\DirectionRepository;
use Doctrine\ORM\Mapping as ORM;
use ApiPlatform\Core\Annotation\ApiResource;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Annotation\ApiFilter;

/**
 * @ApiResource(
 *   normalizationContext={"groups"={"read:directions"}},
 *   collectionOperations={"get"},
 *   itemOperations={"get"}
 * )
 *  @ApiFilter(
 *   SearchFilter::class,
 *   properties={"autorite":"exact",}
 * ),
 * @ORM\Entity(repositoryClass=DirectionRepository::class)
 */
class Direction
{
    /**
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     * @Groups({"read:offres","read:directions","write:offres"})
     */
    private $id;

    /**
     * @Groups({"read:offres","read:directions"})
     * @ORM\Column(type="string", length=255)
     */
    private $libelle;

    /**
     * @Groups({"read:offres","read:directions"})
     * @ORM\ManyToOne(targetEntity=AutoriteContractante::class, inversedBy="directions")
     */
    private $autorite;

    /**
     * @Groups({"read:directions"})
     * @ORM\OneToMany(targetEntity=AppelOffre::class, mappedBy="direction")
     */
    private $appelOffres;

    public function __construct()
    {
        $this->appelOffres = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getLibelle(): ?string
    {
        return $this->libelle;
    }

    public function setLibelle(string $libelle): self
    {
        $this->libelle = $libelle;

        return $this;
    }

    public function getAutorite(): ?AutoriteContractante
    {
        return $this->autorite;
    }

    public function setAutorite(?AutoriteContractante $autorite): self
    {
        $this->autorite = $autorite;

        return $this;
    }

    /**
     * @return Collection|AppelOffre[]
     */
    public function getAppelOffres(): Collection
    {
        return $this->appelOffres;
    }

    public function addAppelOffre(AppelOffre $appelOffre): self
    {
        if (!$this->appelOffres->contains($appelOffre)) {
            $this->appelOffres[] = $appelOffre;
            $appelOffre->setDirection($this);
        }

        return $this;
    }

    public function removeAppelOffre(AppelOffre $appelOffre): self
    {
        if ($this->appelOffres->removeElement($appelOffre)) {
            // set the owning side to null (unless already changed)
            if ($appelOffre->getDirection() === $this) {
                $appelOffre->setDirection(null);
            }
        }

        return $this;
    }
}
