<?php

namespace App\Entity;

use ApiPlatform\Core\Annotation\ApiResource;
use App\Repository\FavorisRepository;
use Doctrine\ORM\Mapping as ORM;
use App\Controller\AddFavorisController;
use Symfony\Component\Serializer\Annotation\Groups;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Annotation\ApiFilter;


/**
 * @ApiResource(
 *   normalizationContext={"groups"={"read:favoris"}},
 *   collectionOperations={
 *         "post"={
 *             "controller"=AddFavorisController::class,
 *             "deserialize"=false,
 *             "path"="/favoris/ajout",
 *             "validation_groups"={"Default", "write:favoris"},
 *             "openapi_context"={
 *                 "requestBody"={
 *                     "content"={
 *                         "multipart/form-data"
 *                     }
 *                 }
 *             }
 *         },
 *         "get"
 *   },  
 * ),
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={"abonne.id":"exact","offre.id":"exact"}
 * ),
 * @ORM\Entity(repositoryClass=FavorisRepository::class)
 */
class Favoris
{
    /**
     * @Groups({"read:favoris"})
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     */
    private $id;

    /**
     * @Groups({"write:favoris","read:favoris"})
     * @ORM\ManyToOne(targetEntity=AppelOffre::class, inversedBy="favoris")
     */
    private $offre;

    /**
     * @Groups({"write:favoris","read:favoris"})
     * @ORM\ManyToOne(targetEntity=Abonne::class, inversedBy="favoris")
     */
    private $abonne;

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getOffre(): ?AppelOffre
    {
        return $this->offre;
    }

    public function setOffre(?AppelOffre $offre): self
    {
        $this->offre = $offre;

        return $this;
    }

    public function getAbonne(): ?Abonne
    {
        return $this->abonne;
    }

    public function setAbonne(?Abonne $abonne): self
    {
        $this->abonne = $abonne;

        return $this;
    }
}
