<?php

namespace App\Entity;

use Symfony\Component\Serializer\Annotation\Groups;
use App\Repository\TypeAutoriteRepository;
use ApiPlatform\Core\Annotation\ApiResource;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ApiResource(
 *   normalizationContext={"groups"={"read:tautorite"}},
 *   collectionOperations={"get"},
 *   itemOperations={"get"}
 * )
 * @ORM\Entity(repositoryClass=TypeAutoriteRepository::class)
 */
class TypeAutorite
{
    /**
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     * @Groups({"read:tautorite","read:offres","read:autorite"})
     */
    private $id;

    /**
     * @ORM\Column(type="string", length=100)
     * @Groups({"read:offres","read:tautorite"})
     */
    private $libelle;

    /**
     * @Groups({"read:tautorite"})
     * @ORM\OneToMany(targetEntity=AutoriteContractante::class, mappedBy="type_autorite")
     */
    private $autoriteContractantes;

    public function __construct()
    {
        $this->autoriteContractantes = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getLibelle(): ?string
    {
        return $this->libelle;
    }

    public function setLibelle(string $libelle): self
    {
        $this->libelle = $libelle;

        return $this;
    }

    /**
     * @return Collection|AutoriteContractante[]
     */
    public function getAutoriteContractantes(): Collection
    {
        return $this->autoriteContractantes;
    }

    public function addAutoriteContractante(AutoriteContractante $autoriteContractante): self
    {
        if (!$this->autoriteContractantes->contains($autoriteContractante)) {
            $this->autoriteContractantes[] = $autoriteContractante;
            $autoriteContractante->setTypeAutorite($this);
        }

        return $this;
    }

    public function removeAutoriteContractante(AutoriteContractante $autoriteContractante): self
    {
        if ($this->autoriteContractantes->removeElement($autoriteContractante)) {
            // set the owning side to null (unless already changed)
            if ($autoriteContractante->getTypeAutorite() === $this) {
                $autoriteContractante->setTypeAutorite(null);
            }
        }

        return $this;
    }
}
