<?php

namespace App\Entity;

use Symfony\Component\Serializer\Annotation\Groups;
use App\Repository\UserRepository;
use ApiPlatform\Core\Annotation\ApiResource;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Security\Core\User\UserInterface;
use ApiPlatform\Core\Bridge\Doctrine\Orm\Filter\SearchFilter;
use ApiPlatform\Core\Annotation\ApiFilter;



/**
 * @ApiResource (
 *     normalizationContext={"groups"={"user:read"}},
 *     denormalizationContext={"groups"={"user:write"}}
 * ),
 * @ApiFilter(
 *   SearchFilter::class,
 *   properties={"email":"exact"}
 * ),
 * @ORM\Entity(repositoryClass=UserRepository::class)
 * @ORM\Table(name="`user`")
 */
class User implements UserInterface
{
    /**
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     * @Groups("user:read")
     */
    private $id;

    /**
     * @Groups("user:read","user:write","read:offres","read:autorite")
     * @ORM\Column(type="string", length=180, unique=true)
     */
    private $email;
 
    /**
     * @Groups("user:read","user:write")
     * @ORM\Column(type="json")
     */
    private $roles = [];

    /**
     * @var string The hashed password
     * @ORM\Column(type="string")
     */
    private $password;

    /**
     * @Groups("user:write")
     */
    private $plainPassword;

    /**
     * @Groups("user:read","user:write")
     * @ORM\OneToMany(targetEntity=Abonne::class, mappedBy="user_id")
     */
    private $abonnes;

    /**
     * @Groups("user:read","user:write")
     * @ORM\Column(type="string", length=15, nullable=true)
     */
    private $nom;

    /**
     * @Groups("user:read","user:write")
     * @ORM\Column(type="string", length=100, nullable=true)
     */
    private $prenom;

    /**
     * @Groups("user:write")
     * 
     */
    private $organisation;

    /**
     * @Groups("user:read","user:write")
     * @ORM\OneToMany(targetEntity=AutoriteContractante::class, mappedBy="user")
     */
    private $autoriteContractantes;

    /**
     * @Groups("user:write")
     */
    private $type;


    public function __construct()
    {
        $this->abonnes = new ArrayCollection();
        $this->autoriteContractantes = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getEmail(): ?string
    {
        return $this->email;
    }

    public function setEmail(string $email): self
    {
        $this->email = $email;

        return $this;
    }

    /**
     * A visual identifier that represents this user.
     *
     * @see UserInterface
     */
    public function getUsername(): string
    {
        return (string) $this->email;
    }

    /**
     * @see UserInterface
     */
    public function getRoles(): array
    {
        $roles = $this->roles;
        // guarantee every user at least has ROLE_USER
       /*  $roles[] = 'ROLE_USER'; */

        return array_unique($roles);
    }

    public function setRoles(array $roles): self
    {
        $this->roles = $roles;

        return $this;
    }

    /**
     * @see UserInterface
     */
    public function getPassword(): string
    {
        return (string) $this->password;
    }

    public function setPassword(string $password): self
    {
        $this->password = $password;

        return $this;
    }


    /**
     * Returning a salt is only needed, if you are not using a modern
     * hashing algorithm (e.g. bcrypt or sodium) in your security.yaml.
     *
     * @see UserInterface
     */
    public function getSalt(): ?string
    {
        return null;
    }

    /**
     * @see UserInterface
     */
    public function eraseCredentials()
    {
        $this->plainPassword = null;
    }

    /**
     * @return Collection|Abonne[]
     */
    public function getAbonnes(): Collection
    {
        return $this->abonnes;
    }

    public function addAbonne(Abonne $abonne): self
    {
        if (!$this->abonnes->contains($abonne)) {
            $this->abonnes[] = $abonne;
            $abonne->setUserId($this);
        }
        return $this;
    }

    public function removeAbonne(Abonne $abonne): self
    {
        if ($this->abonnes->removeElement($abonne)) {
            // set the owning side to null (unless already changed)
            if ($abonne->getUserId() === $this) {
                $abonne->setUserId(null);
            }
        }

        return $this;
    }

    public function getPlainPassword(): ?string
    {
        return $this->plainPassword;
    }

    public function setPlainPassword(string $plainPassword): self
    {
        $this->plainPassword = $plainPassword;

        return $this;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function setNom(?string $nom): self
    {
        $this->nom = $nom;

        return $this;
    }

    public function getPrenom(): ?string
    {
        return $this->prenom;
    }

    public function setPrenom(?string $prenom): self
    {
        $this->prenom = $prenom;

        return $this;
    }

    public function getOrganisation(): ?string
    {
        return $this->organisation;
    }

    public function setOrganisation(string $organisation): self
    {
        $this->organisation = $organisation;

        return $this;
    }

    /**
     * @return Collection|AutoriteContractante[]
     */
    public function getAutoriteContractantes(): Collection
    {
        return $this->autoriteContractantes;
    }

    public function addAutoriteContractante(AutoriteContractante $autoriteContractante): self
    {
        if (!$this->autoriteContractantes->contains($autoriteContractante)) {
            $this->autoriteContractantes[] = $autoriteContractante;
            $autoriteContractante->setUser($this);
        }

        return $this;
    }

    public function removeAutoriteContractante(AutoriteContractante $autoriteContractante): self
    {
        if ($this->autoriteContractantes->removeElement($autoriteContractante)) {
            // set the owning side to null (unless already changed)
            if ($autoriteContractante->getUser() === $this) {
                $autoriteContractante->setUser(null);
            }
        }

        return $this;
    }

    public function getType(): ?int
    {
        return $this->type;
    }

    public function setType(?int $type): self
    {
        $this->type = $type;

        return $this;
    }

}
